'use strict';

const fs = require('fs');
const module$1 = require('module');
const path = require('path');
const enhancedResolve = require('enhanced-resolve');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

const fs__default = /*#__PURE__*/_interopDefault(fs);
const enhancedResolve__default = /*#__PURE__*/_interopDefault(enhancedResolve);

const create = enhancedResolve__default['default'].create;
const index = (({
  types: t
}, {
  ignoreBuiltins,
  ignoreBareImport,
  ignore = [],
  baseUrl = ".",
  paths = {},
  pathRegexps: optionPathRegexps = {},
  relative: optionRelative = true,
  cache = false,
  ...resolveOptions
}) => {
  // /^\0/ rollup helper
  const ignores = [/^\0/, ...ignore];

  if (ignoreBuiltins) {
    ignores.push(...module$1.builtinModules);
  }

  if (ignoreBareImport) {
    ignores.push(/^@?[A-Za-z]/);
  }

  const basePath = path.resolve(baseUrl);
  const pathEntries = Object.entries(paths).map(([prefix, path]) => [prefix, path]);
  const pathRegexps = Object.entries(optionPathRegexps).map(([regexp, path]) => [new RegExp(regexp), path]);

  if (!cache) {
    resolveOptions.fileSystem = fs__default['default'];
  }

  resolveOptions = {
    conditionNames: ["import"],
    extensions: [".tsx", ".ts", ".jsx", ".mjs", ".js", ".cjs", ".json", ".node"],
    ...resolveOptions
  };
  const enhancedResolve = create.sync(resolveOptions);

  const resolve = (path$1, request, deps) => {
    if (ignores.some(ignore => typeof ignore === "string" ? ignore === request : ignore.test(request))) {
      deps.push(request);
      return request;
    }

    for (const [prefix, path$1] of pathEntries) {
      if (request.startsWith(prefix)) {
        request = path.resolve(basePath, path$1 + request.slice(prefix.length));
        break;
      }
    }

    for (const [regexp, path$1] of pathRegexps) {
      if (regexp.test(request)) {
        request = path.resolve(basePath, request.replace(regexp, path$1));
        break;
      }
    }

    const dir = path.dirname(path$1);
    request = enhancedResolve(dir, request) || request;
    deps.push(request);

    if (optionRelative) {
      request = path.relative(dir, request);

      if (request[0] !== "/" && request[0] !== ".") {
        request = "./" + request;
      }
    }

    return request;
  };

  const transformDeclaration = (path, state) => {
    const {
      source
    } = path.node;

    if (!state.filename || !source) {
      return;
    }

    let modulePath = source.value;

    try {
      modulePath = resolve(state.filename, modulePath, state.file.metadata.deps);
    } catch (e) {
      console.log(path.buildCodeFrameError(e?.message));
      return;
    }

    const sourcePath = path.get("source");

    if (Array.isArray(sourcePath)) {
      return;
    }

    sourcePath.replaceWith(t.stringLiteral(modulePath));
  };

  const transformImport = (path, state) => {
    if (!state.filename || !path.parentPath.isCallExpression()) {
      return;
    }

    const arg = path.parentPath.get("arguments")[0];
    let modulePath;

    if (arg.isTemplateLiteral() && arg.node.expressions.length === 0) {
      modulePath = arg.node.quasis[0].value.cooked;
    }

    if (arg.isStringLiteral()) {
      modulePath = arg.node.value;
    }

    if (!modulePath) {
      return;
    }

    try {
      modulePath = resolve(state.filename, modulePath, state.file.metadata.deps);
    } catch (e) {
      console.log(path.buildCodeFrameError(e?.message));
      return;
    }

    arg.replaceWith(t.stringLiteral(modulePath));
  };

  return {
    name: "resolve",

    pre(_state) {
      this.file.metadata.deps = [];
    },

    post() {
      this.file.metadata.deps = [...new Set(this.file.metadata.deps)];
    },

    visitor: {
      ImportDeclaration: transformDeclaration,
      ExportAllDeclaration: transformDeclaration,
      ExportNamedDeclaration: transformDeclaration,
      Import: transformImport
    }
  };
});

module.exports = index;
//# sourceMappingURL=index.js.map
